@@ -13,10 +13,17 @@ can:
 
 * read field constructs from netCDF, CDL, PP and UM datasets,
 
+* read field constructs and domain constructs from netCDF, CDL, PP and
+  UM datasets with a choice of netCDF backends,
+
+* read files from OPeNDAP servers and S3 object stores,
+
 * create new field constructs in memory,
 
 * write and append field constructs to netCDF datasets on disk,
 
+* read, write, and manipulate UGRID mesh topologies,
+
 * read, write, and create coordinates defined by geometry cells,
 
 * read netCDF and CDL datasets containing hierarchical groups,
@@ -74,8 +81,8 @@ installation and source code.
 """
 
 __Conventions__ = "CF-1.11"
-__date__ = "2024-04-26"
-__version__ = "3.16.2"
+__date__ = "2025-01-28"
+__version__ = "3.16.3"
 
 _requires = (
     "numpy",
@@ -116,7 +123,7 @@ except ImportError as error1:
     raise ImportError(_error0 + str(error1))
 
 try:
-    import numpy
+    import numpy as np
 except ImportError as error1:
     raise ImportError(_error0 + str(error1))
 
@@ -184,10 +191,11 @@ if Version(cftime.__version__) < Version(_minimum_vn):
 
 # Check the version of numpy
 _minimum_vn = "1.22"
-if Version(numpy.__version__) < Version(_minimum_vn):
-    raise RuntimeError(
-        f"Bad numpy version: cf requires numpy>={_minimum_vn}. "
-        f"Got {numpy.__version__} at {numpy.__file__}"
+_maximum_vn = "2.0"
+if not Version(_minimum_vn) <= Version(np.__version__) < Version(_maximum_vn):
+    raise ValueError(
+        "Bad numpy version: cf requires _minimum_vn}<=numpy<{_maximum_vn}. "
+        f"Got {np.__version__} at {np.__file__}"
     )
 
 # Check the version of cfunits
@@ -199,20 +207,29 @@ if Version(cfunits.__version__) < Version(_minimum_vn):
     )
 
 # Check the version of cfdm
-_minimum_vn = "1.11.1.0"
-_maximum_vn = "1.11.2.0"
-_cfdm_version = Version(cfdm.__version__)
-if not Version(_minimum_vn) <= _cfdm_version < Version(_maximum_vn):
+_minimum_vn = "1.11.2.0"
+_maximum_vn = "1.11.3.0"
+if (
+    not Version(_minimum_vn)
+    <= Version(cfdm.__version__)
+    < Version(_maximum_vn)
+):
     raise RuntimeError(
         f"Bad cfdm version: cf requires {_minimum_vn}<=cfdm<{_maximum_vn}. "
-        f"Got {_cfdm_version} at {cfdm.__file__}"
+        f"Got {cfdm.__version__} at {cfdm.__file__}"
     )
 
 # Check the version of dask
-_minimum_vn = "2022.12.1"
-if Version(dask.__version__) < Version(_minimum_vn):
-    raise RuntimeError(
-        f"Bad dask version: cf requires dask>={_minimum_vn}. "
+
+_minimum_vn = "2024.6.1"
+_maximum_vn = "2024.7.1"
+if (
+    not Version(_minimum_vn)
+    <= Version(dask.__version__)
+    <= Version(_maximum_vn)
+):
+    raise ValueError(
+        "Bad dask version: cf requires {_minimum_vn}<=dask<={_maximum_vn}. "
         f"Got {dask.__version__} at {dask.__file__}"
     )
 
@@ -232,6 +249,8 @@ if Version(scipy.__version__) < Version(_minimum_vn):
         f"Got {scipy.__version__} at {scipy.__file__}"
     )
 
+del _minimum_vn, _maximum_vn
+
 from .constructs import Constructs
 
 from .mixin import Coordinate
@@ -274,15 +293,19 @@ from .data import Data
 from .data.array import (
     BoundsFromNodesArray,
     CellConnectivityArray,
-    CFANetCDFArray,
+    CFAH5netcdfArray,
+    CFANetCDF4Array,
     FullArray,
     GatheredArray,
+    H5netcdfArray,
     NetCDFArray,
+    NetCDF4Array,
     PointTopologyArray,
     RaggedContiguousArray,
     RaggedIndexedArray,
     RaggedIndexedContiguousArray,
     SubsampledArray,
+    UMArray,
 )
 
 from .data.fragment import (
