@@ -5,8 +5,9 @@ import cftime
 import numpy as np
 
 from .functions import _DEPRECATION_ERROR_CLASS
+from .functions import size as cf_size
 
-_default_calendar = "gregorian"
+default_calendar = "gregorian"
 
 # --------------------------------------------------------------------
 # Mapping of CF calendars to cftime date-time objects
@@ -21,6 +22,21 @@ _datetime_object = {
     ("julian",): cftime.DatetimeJulian,
 }
 
+canonical_calendar = {
+    None: "standard",
+    "gregorian": "standard",
+    "standard": "standard",
+    "proleptic_gregorian": "proleptic_gregorian",
+    "julian": "julian",
+    "noleap": "noleap",
+    "365_day": "noleap",
+    "all_366_day": "all_leap",
+    "all_leap": "all_leap",
+    "": "",
+    "none": "",
+}
+
+
 _calendar_map = {None: "gregorian"}
 
 
@@ -45,10 +61,12 @@ class Datetime(cftime.datetime):
         dayofyr=1,
         calendar=None,
     ):
-        """TODO."""
+        """**Initialisation**"""
         _DEPRECATION_ERROR_CLASS(
             "Datetime",
             "Use function 'cf.dt' to create date-time objects instead.",
+            version="3.0.0",
+            removed_at="4.0.0",
         )  # pragma: no cover
 
 
@@ -76,7 +94,7 @@ def dt(
             * A `str` specifying an ISO 8601-like date-time string (in
               which non-Gregorian calendar dates are allowed).
 
-            * `datetime.datetime or `cftime.datetime`. A new date-time
+            * `datetime.datetime` or `cftime.datetime`. A new date-time
               object is returned for the given date-time.
 
 
@@ -92,7 +110,7 @@ def dt(
         `cftime.datetime`
             The new date-time object.
 
-    **Examples:**
+    **Examples**
 
     >>> d = cf.dt(2003)
     >>> d
@@ -138,7 +156,7 @@ def dt(
         (year, month, day, hour, minute, second) = arg.timetuple()[:6]
         microsecond = arg.microsecond
         if calendar == "":
-            calendar = _default_calendar
+            calendar = default_calendar
 
     else:
         year = arg
@@ -197,7 +215,7 @@ def dt_vector(
         `numpy.ndarray`
             1-d array of date-time objects.
 
-    **Examples:**
+    **Examples**
 
     TODO
 
@@ -224,7 +242,7 @@ def dt_vector(
         )
 
     sizes = set(
-        map(np.size, (arg, month, day, hour, minute, second, microsecond))
+        map(cf_size, (arg, month, day, hour, minute, second, microsecond))
     )
 
     if len(sizes) == 1 and 1 in sizes:
@@ -317,7 +335,7 @@ def st2dt(array, units_in=None, dummy0=None, dummy1=None):
             An array of `cftime.datetime` objects with the same shape
             as *array*.
 
-    **Examples:**
+    **Examples**
 
     """
     func = partial(st2datetime, calendar=units_in._calendar)
@@ -386,22 +404,16 @@ def st2elements(date_string):
     if utc_offset:
         raise ValueError("Can't specify a time offset from UTC")
 
-    return (
-        year,
-        month,
-        day,
-        hour,
-        minute,
-        second,
-        microsecond,
-    )
+    return (year, month, day, hour, minute, second, microsecond)
 
 
 def rt2dt(array, units_in, units_out=None, dummy1=None):
-    """Convert reference times  to date-time objects.
+    """Convert reference times to date-time objects.
 
     The returned array is always independent.
 
+    .. seealso:: `dt2rt`
+
     :Parameters:
 
         array: numpy array-like
@@ -420,6 +432,17 @@ def rt2dt(array, units_in, units_out=None, dummy1=None):
             An array of `cftime.datetime` objects with the same shape
             as *array*.
 
+    **Examples**
+
+    >>> print(
+    ...   cf.cfdatetime.rt2dt(
+    ...     np.ma.array([0, 685.5], mask=[True, False]),
+    ...     units_in=cf.Units('days since 2000-01-01')
+    ...   )
+    ... )
+    [--
+     cftime.DatetimeGregorian(2001, 11, 16, 12, 0, 0, 0, has_year_zero=False)]
+
     """
     ndim = np.ndim(array)
     if not ndim and np.ma.is_masked(array):
@@ -434,6 +457,9 @@ def rt2dt(array, units_in, units_out=None, dummy1=None):
         array, units, calendar, only_use_cftime_datetimes=True
     )
 
+    if not isinstance(array, np.ndarray):
+        array = np.array(array, dtype=object)
+
     return array
 
 
@@ -446,20 +472,23 @@ def dt2Dt(x, calendar=None):
 
 
 def dt2rt(array, units_in, units_out, dummy1=None):
-    """Round to the nearest millisecond. This is only necessary whilst
-    netCDF4 time functions have an accuracy of no better than 1
-    millisecond (which is still the case at version 1.2.2).
+    """Return numeric time values from datetime objects.
 
-    The returned array is always independent.
+    .. seealso:: `rt2dt`
 
     :Parameters:
 
         array: numpy array-like of date-time objects
+            The datetime objects must be in UTC with no time-zone
+            offset.
 
-        units_in:
+      units_in:
             Ignored.
 
         units_out: `Units`
+            The units of the numeric time values. If there is a
+            time-zone offset in *units_out*, it will be applied to the
+            returned numeric values.
 
         dummy1:
             Ignored.
@@ -469,16 +498,29 @@ def dt2rt(array, units_in, units_out, dummy1=None):
         `numpy.ndarray`
             An array of numbers with the same shape as *array*.
 
+    **Examples**
+
+    >>> print(
+    ...   cf.cfdatetime.dt2rt(
+    ...     np.ma.array([0, cf.dt('2001-11-16 12:00')], mask=[True, False]),
+    ...     None,
+    ...     units_out=cf.Units('days since 2000-01-01')
+    ...   )
+    ... )
+    [-- 685.5]
+
     """
-    ndim = np.ndim(array)
+    isscalar = not np.ndim(array)
 
-    #    array = units_out._utime.date2num(array)
     array = cftime.date2num(
         array, units=units_out.units, calendar=units_out._utime.calendar
     )
 
-    if not ndim:
-        array = np.asanyarray(array)
+    if isscalar:
+        if array is np.ma.masked:
+            array = np.ma.masked_all(())
+        else:
+            array = np.asanyarray(array)
 
     return array
 
