@@ -1,9 +1,12 @@
 from .boundsfromnodesarray import BoundsFromNodesArray
 from .cellconnectivityarray import CellConnectivityArray
-from .cfanetcdfarray import CFANetCDFArray
+from .cfah5netcdfarray import CFAH5netcdfArray
+from .cfanetcdf4array import CFANetCDF4Array
 from .fullarray import FullArray
 from .gatheredarray import GatheredArray
+from .h5netcdfarray import H5netcdfArray
 from .netcdfarray import NetCDFArray
+from .netcdf4array import NetCDF4Array
 from .pointtopologyarray import PointTopologyArray
 from .raggedcontiguousarray import RaggedContiguousArray
 from .raggedindexedarray import RaggedIndexedArray
