@@ -1,11 +1,13 @@
 import numpy as np
+from cfdm.data.mixin import IndexMixin
 
+from ...functions import indices_shape, parse_indices
 from .abstract import Array
 
 _FULLARRAY_HANDLED_FUNCTIONS = {}
 
 
-class FullArray(Array):
+class FullArray(IndexMixin, Array):
     """A array filled with a given value.
 
     The array may be empty or all missing values.
@@ -19,8 +21,7 @@ class FullArray(Array):
         fill_value=None,
         dtype=None,
         shape=None,
-        units=False,
-        calendar=False,
+        attributes=None,
         source=None,
         copy=True,
     ):
@@ -38,23 +39,22 @@ class FullArray(Array):
             shape: `tuple`
                 The array dimension sizes.
 
-            units: `str` or `None`, optional
-                The units of the netCDF variable. Set to `None` to
-                indicate that there are no units. If unset then the
-                units will be set to `None` during the first
-                `__getitem__` call.
+            {{init attributes: `dict` or `None`, optional}}
 
-            calendar: `str` or `None`, optional
-                The calendar of the netCDF variable. By default, or if
-                set to `None`, then the CF default calendar is
-                assumed, if applicable. If unset then the calendar
-                will be set to `None` during the first `__getitem__`
-                call.
+                .. versionadded:: 1.11.2.0
 
             {{init source: optional}}
 
             {{init copy: `bool`, optional}}
 
+            units: `str` or `None`, optional
+                Deprecated at version 1.11.2.0. Use the
+                *attributes* parameter instead.
+
+            calendar: `str` or `None`, optional
+                Deprecated at version 1.11.2.0. Use the
+                *attributes* parameter instead.
+
         """
         super().__init__(source=source, copy=copy)
 
@@ -75,42 +75,14 @@ class FullArray(Array):
                 shape = None
 
             try:
-                units = source._get_component("units", False)
+                attributes = source._get_component("attributes", False)
             except AttributeError:
-                units = False
-
-            try:
-                calendar = source._get_component("calendar", None)
-            except AttributeError:
-                calendar = None
+                attributes = None
 
         self._set_component("full_value", fill_value, copy=False)
         self._set_component("dtype", dtype, copy=False)
         self._set_component("shape", shape, copy=False)
-        self._set_component("units", units, copy=False)
-        self._set_component("calendar", calendar, copy=False)
-
-    def __array__(self, *dtype):
-        """The numpy array interface.
-
-        .. versionadded:: 3.15.0
-
-        :Parameters:
-
-            dtype: optional
-                Typecode or data-type to which the array is cast.
-
-        :Returns:
-
-            `numpy.ndarray`
-                An independent numpy array of the data.
-
-        """
-        array = self[...]
-        if not dtype:
-            return array
-        else:
-            return array.astype(dtype[0], copy=False)
+        self._set_component("attributes", attributes, copy=False)
 
     def __array_function__(self, func, types, args, kwargs):
         """The `numpy` `__array_function__` protocol.
@@ -128,54 +100,6 @@ class FullArray(Array):
 
         return _FULLARRAY_HANDLED_FUNCTIONS[func](*args, **kwargs)
 
-    def __getitem__(self, indices):
-        """x.__getitem__(indices) <==> x[indices]
-
-        Returns a numpy array.
-
-        """
-        # If 'apply_indices' is False then we can make a filled numpy
-        # array that already has the correct shape. Otherwise we need
-        # to make one with shape 'self.shape' and then apply the
-        # indices to that.
-        apply_indices = False
-
-        if indices is Ellipsis:
-            array_shape = self.shape
-        else:
-            array_shape = []
-            for i, size in zip(indices, self.shape):
-                if not isinstance(i, slice):
-                    continue
-
-                start, stop, step = i.indices(size)
-                a, b = divmod(stop - start, step)
-                if b:
-                    a += 1
-
-                array_shape.append(a)
-
-            if len(array_shape) != self.ndim:
-                apply_indices = True
-                array_shape = self.shape
-
-        fill_value = self.get_full_value()
-        if fill_value is np.ma.masked:
-            array = np.ma.masked_all(array_shape, dtype=self.dtype)
-        elif fill_value is not None:
-            array = np.full(
-                array_shape, fill_value=fill_value, dtype=self.dtype
-            )
-        else:
-            array = np.empty(array_shape, dtype=self.dtype)
-
-        if apply_indices:
-            array = self.get_subspace(array, indices)
-
-        self._set_units()
-
-        return array
-
     def __repr__(self):
         """Called by the `repr` built-in function.
 
@@ -196,6 +120,53 @@ class FullArray(Array):
 
         return f"Filled with {fill_value!r}"
 
+    def _get_array(self, index=None):
+        """Returns the full array.
+
+        .. versionadded:: 1.11.2.0
+
+        .. seealso:: `__array__`, `index`
+
+        :Parameters:
+
+            {{index: `tuple` or `None`, optional}}
+
+        :Returns:
+
+            `numpy.ndarray`
+                The subspace.
+
+        """
+        if index is None:
+            shape = self.shape
+        else:
+            original_shape = self.original_shape
+            index = parse_indices(original_shape, index, keepdims=False)
+            shape = indices_shape(index, original_shape, keepdims=False)
+
+        fill_value = self.get_full_value()
+        if fill_value is np.ma.masked:
+            array = np.ma.masked_all(shape, dtype=self.dtype)
+        elif fill_value is not None:
+            array = np.full(shape, fill_value=fill_value, dtype=self.dtype)
+        else:
+            array = np.empty(shape, dtype=self.dtype)
+
+        return array
+
+    @property
+    def array(self):
+        """Return an independent numpy array containing the data.
+
+        .. versionadded:: 1.11.2.0
+
+        :Returns:
+
+            `numpy.ndarray`
+                An independent numpy array of the data.
+        """
+        return np.asanyarray(self)
+
     @property
     def dtype(self):
         """Data-type of the data elements."""
