@@ -1,3 +1,5 @@
+from .activestoragemixin import ActiveStorageMixin
 from .arraymixin import ArrayMixin
+from .cfamixin import CFAMixin
 from .compressedarraymixin import CompressedArrayMixin
 from .filearraymixin import FileArrayMixin
