@@ -1,6 +1,4 @@
 import cfdm
-import netCDF4
-import numpy as np
 from packaging.version import Version
 
 
@@ -154,6 +152,7 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         parent_ncvar=None,
         coord_ncvar=None,
         cfa_term=None,
+        compression_index=False,
     ):
         """Create data for a netCDF or CFA-netCDF variable.
 
@@ -181,6 +180,12 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
 
                 .. versionadded:: 3.15.0
 
+           compression_index: `bool`, optional
+                True if the data being created are compression
+                indices.
+
+                .. versionadded:: 3.15.2
+
         :Returns:
 
             `Data`
@@ -202,13 +207,6 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             if data.npartitions == 1:
                 data._cfa_set_write(True)
 
-            if self.implementation.get_construct_type(construct) != "field":
-                # Only cache values from non-field data, on the
-                # assumption that field data is, in general, so large
-                # that finding the cached values takes too long. See
-                # method `_cache_data_elements` for details.
-                self._cache_data_elements(data, ncvar)
-
             return data
 
         # ------------------------------------------------------------
@@ -237,17 +235,18 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 coord_ncvar=coord_ncvar,
             )
 
+        attributes = kwargs["attributes"]
         data = self._create_Data(
             cfa_array,
             ncvar,
-            units=kwargs["units"],
-            calendar=kwargs["calendar"],
+            units=attributes.get("units"),
+            calendar=attributes.get("calendar"),
         )
 
         # Note: We don't cache elements from CFA variables, because
         #       the data are in fragment files which have not been
-        #       opened; and may not not even be openable, such as
-        #       could be the case if a fragement was on tape storage.
+        #       opened and may not not even be openable (such as could
+        #       be the case if a fragment file was on tape storage).
 
         # Set the CFA write status to True iff each non-aggregated
         # axis has exactly one dask storage chunk
@@ -297,99 +296,6 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             and ncvar not in g["external_variables"]
         )
 
-    def _create_Data(
-        self,
-        array,
-        ncvar,
-        units=None,
-        calendar=None,
-        ncdimensions=(),
-        **kwargs,
-    ):
-        """Create a Data object from a netCDF variable.
-
-        .. versionadded:: 3.0.0
-
-        :Parameters:
-
-            array: `Array`
-                The file array.
-
-            ncvar: `str`
-                The netCDF variable containing the array.
-
-            units: `str`, optional
-                The units of *array*. By default, or if `None`, it is
-                assumed that there are no units.
-
-            calendar: `str`, optional
-                The calendar of *array*. By default, or if `None`, it is
-                assumed that there is no calendar.
-
-            ncdimensions: sequence of `str`, optional
-                The netCDF dimensions spanned by the array.
-
-                .. versionadded:: 3.14.0
-
-            kwargs: optional
-                Extra parameters to pass to the initialisation of the
-                returned `Data` object.
-
-        :Returns:
-
-            `Data`
-
-        """
-        if array.dtype is None:
-            # The array is based on a netCDF VLEN variable, and
-            # therefore has unknown data type. To find the correct
-            # data type (e.g. "<U7"), we need to read the entire array
-            # from its netCDF variable into memory to find the longest
-            # string.
-            g = self.read_vars
-            if g["has_groups"]:
-                group, name = self._netCDF4_group(
-                    g["variable_grouped_dataset"][ncvar], ncvar
-                )
-                variable = group.variables.get(name)
-            else:
-                variable = g["variables"].get(ncvar)
-
-            array = variable[...]
-
-            string_type = isinstance(array, str)
-            if string_type:
-                # A netCDF string type scalar variable comes out as Python
-                # str object, so convert it to a numpy array.
-                array = np.array(array, dtype=f"U{len(array)}")
-
-            if not variable.ndim:
-                # NetCDF4 has a thing for making scalar size 1
-                # variables into 1d arrays
-                array = array.squeeze()
-
-            if not string_type:
-                # A N-d (N>=1) netCDF string type variable comes out
-                # as a numpy object array, so convert it to numpy
-                # string array.
-                array = array.astype("U", copy=False)
-                # NetCDF4 doesn't auto-mask VLEN variables
-                array = np.ma.where(array == "", np.ma.masked, array)
-
-        # Parse dask chunks
-        chunks = self._parse_chunks(ncvar)
-
-        data = super()._create_Data(
-            array,
-            ncvar,
-            units=units,
-            calendar=calendar,
-            chunks=chunks,
-            **kwargs,
-        )
-
-        return data
-
     def _customise_read_vars(self):
         """Customise the read parameters.
 
@@ -450,159 +356,6 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             for term_ncvar in parsed_aggregated_data.values():
                 g["do_not_create_field"].add(term_ncvar)
 
-    def _cache_data_elements(self, data, ncvar):
-        """Cache selected element values.
-
-        Updates *data* in-place to store its first, second,
-        penultimate, and last element values (as appropriate).
-
-        These values are used by, amongst other things,
-        `cf.Data.equals`, `cf.aggregate` and for inspection.
-
-        Doing this here is quite cheap because only the individual
-        elements are read from the already-open file, as opposed to
-        being retrieved from *data* (which would require a whole dask
-        chunk to be read to get each single value).
-
-        However, empirical evidence shows that using netCDF4 to access
-        the first and last elements of a large array on disk
-        (e.g. shape (1, 75, 1207, 1442)) is slow (e.g. ~2 seconds) and
-        doesn't scale well with array size (i.e. it takes
-        disproportionally longer for larger arrays). Such arrays are
-        usually in field constructs, for which `cf.aggregate` does not
-        need to know any array values, so this method should be used
-        with caution, if at all, on field construct data.
-
-        .. versionadded:: 3.14.0
-
-        :Parameters:
-
-            data: `Data`
-                The data to be updated with its cached values.
-
-            ncvar: `str`
-                The name of the netCDF variable that contains the
-                data.
-
-        :Returns:
-
-            `None`
-
-        """
-        if data.data.get_compression_type():
-            # Don't get cached elements from arrays compressed by
-            # convention, as they'll likely be wrong.
-            return
-
-        g = self.read_vars
-
-        # Get the netCDF4.Variable for the data
-        if g["has_groups"]:
-            group, name = self._netCDF4_group(
-                g["variable_grouped_dataset"][ncvar], ncvar
-            )
-            variable = group.variables.get(name)
-        else:
-            variable = g["variables"].get(ncvar)
-
-        # Get the required element values
-        size = data.size
-        ndim = data.ndim
-
-        char = False
-        if variable.ndim == ndim + 1:
-            dtype = variable.dtype
-            if dtype is not str and dtype.kind in "SU":
-                # This variable is a netCDF classic style char array
-                # with a trailing dimension that needs to be collapsed
-                char = True
-
-        if ndim == 1:
-            # Also cache the second element for 1-d data, on the
-            # assumption that they may well be dimension coordinate
-            # data.
-            if size == 1:
-                indices = (0, -1)
-                value = variable[...]
-                values = (value, value)
-            elif size == 2:
-                indices = (0, 1, -1)
-                value = variable[-1:]
-                values = (variable[:1], value, value)
-            else:
-                indices = (0, 1, -1)
-                values = (variable[:1], variable[1:2], variable[-1:])
-        elif ndim == 2 and data.shape[-1] == 2:
-            # Assume that 2-d data with a last dimension of size 2
-            # contains coordinate bounds, for which it is useful to
-            # cache the upper and lower bounds of the the first and
-            # last cells.
-            indices = (0, 1, -2, -1)
-            ndim1 = ndim - 1
-            values = (
-                variable[(slice(0, 1),) * ndim1 + (slice(0, 1),)],
-                variable[(slice(0, 1),) * ndim1 + (slice(1, 2),)],
-            )
-            if data.size == 2:
-                values = values + values
-            else:
-                values += (
-                    variable[(slice(-1, None, 1),) * ndim1 + (slice(0, 1),)],
-                    variable[(slice(-1, None, 1),) * ndim1 + (slice(1, 2),)],
-                )
-        elif size == 1:
-            indices = (0, -1)
-            value = variable[...]
-            values = (value, value)
-        elif size == 3:
-            indices = (0, 1, -1)
-            if char:
-                values = variable[...].reshape(3, variable.shape[-1])
-            else:
-                values = variable[...].flatten()
-        else:
-            indices = (0, -1)
-            values = (
-                variable[(slice(0, 1),) * ndim],
-                variable[(slice(-1, None, 1),) * ndim],
-            )
-
-        # Create a dictionary of the element values
-        elements = {}
-        for index, value in zip(indices, values):
-            if char:
-                # Variable is a netCDF classic style char array, so
-                # collapse (by concatenation) the outermost (fastest
-                # varying) dimension. E.g. [['a','b','c']] becomes
-                # ['abc']
-                if value.dtype.kind == "U":
-                    value = value.astype("S")
-
-                a = netCDF4.chartostring(value)
-                shape = a.shape
-                a = np.array([x.rstrip() for x in a.flat])
-                a = np.reshape(a, shape)
-                value = np.ma.masked_where(a == "", a)
-
-            if np.ma.is_masked(value):
-                value = np.ma.masked
-            else:
-                try:
-                    value = value.item()
-                except (AttributeError, ValueError):
-                    # AttributeError: A netCDF string type scalar
-                    # variable comes out as Python str object, which
-                    # has no 'item' method.
-                    #
-                    # ValueError: A size-0 array can't be converted to
-                    # a Python scalar.
-                    pass
-
-            elements[index] = value
-
-        # Store the elements in the data object
-        data._set_cached_elements(elements)
-
     def _create_cfanetcdfarray(
         self,
         ncvar,
@@ -635,13 +388,13 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         :Returns:
 
             (`CFANetCDFArray`, `dict`)
-                The new `NetCDFArray` instance and dictionary of the
-                kwargs used to create it.
+                The new `CFANetCDFArray` instance and dictionary of
+                the kwargs used to create it.
 
         """
         g = self.read_vars
 
-        # Get the kwargs needed to instantiate a general NetCDFArray
+        # Get the kwargs needed to instantiate a general netCDF array
         # instance
         kwargs = self._create_netcdfarray(
             ncvar,
@@ -650,10 +403,9 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             return_kwargs_only=True,
         )
 
-        # Get rid of the incorrect shape of (). This will end up
-        # getting set correctly by the CFANetCDFArray instance.
+        # Get rid of the incorrect shape. This will end up getting set
+        # correctly by the CFANetCDFArray instance.
         kwargs.pop("shape", None)
-
         aggregated_data = g["cfa_aggregated_data"][ncvar]
 
         standardised_terms = ("location", "file", "address", "format")
@@ -678,7 +430,11 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         kwargs["instructions"] = " ".join(sorted(instructions))
 
         # Use the kwargs to create a CFANetCDFArray instance
-        array = self.implementation.initialise_CFANetCDFArray(**kwargs)
+        if g["original_netCDF4"]:
+            array = self.implementation.initialise_CFANetCDF4Array(**kwargs)
+        else:
+            # h5netcdf
+            array = self.implementation.initialise_CFAH5netcdfArray(**kwargs)
 
         return array, kwargs
 
@@ -713,19 +469,23 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         :Returns:
 
             (`CFANetCDFArray`, `dict`)
-                The new `NetCDFArray` instance and dictionary of the
-                kwargs used to create it.
+                The new `CFANetCDFArray` instance and dictionary of
+                the kwargs used to create it.
 
         """
         g = self.read_vars
 
-        # Get the kwargs needed to instantiate a general NetCDFArray
+        # Get the kwargs needed to instantiate a general netCDF array
         # instance
         kwargs = self._create_netcdfarray(
             ncvar,
             return_kwargs_only=True,
         )
 
+        # Get rid of the incorrect shape. This will end up getting set
+        # correctly by the CFANetCDFArray instance.
+        kwargs.pop("shape", None)
+
         instructions = []
         aggregation_instructions = {}
         for t, term_ncvar in g["cfa_aggregated_data"][parent_ncvar].items():
@@ -740,76 +500,14 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         kwargs["x"] = aggregation_instructions
         kwargs["instructions"] = " ".join(sorted(instructions))
 
-        # Use the kwargs to create a CFANetCDFArray instance
-        array = self.implementation.initialise_CFANetCDFArray(**kwargs)
+        if g["original_netCDF4"]:
+            array = self.implementation.initialise_CFANetCDF4Array(**kwargs)
+        else:
+            # h5netcdf
+            array = self.implementation.initialise_CFAH5netcdfArray(**kwargs)
 
         return array, kwargs
 
-    def _parse_chunks(self, ncvar):
-        """Parse the dask chunks.
-
-        .. versionadded:: 3.14.0
-
-        :Parameters:
-
-            ncvar: `str`
-                The name of the netCDF variable containing the array.
-
-        :Returns:
-
-            `str`, `int` or `dict`
-                The parsed chunks that are suitable for passing to a
-                `Data` object containing the variable's array.
-
-        """
-        g = self.read_vars
-
-        default_chunks = "auto"
-        chunks = g.get("chunks", default_chunks)
-
-        if chunks is None:
-            return -1
-
-        if isinstance(chunks, dict):
-            if not chunks:
-                return default_chunks
-
-            # For ncdimensions = ('time', 'lat'):
-            #
-            # chunks={} -> ["auto", "auto"]
-            # chunks={'ncdim%time': 12} -> [12, "auto"]
-            # chunks={'ncdim%time': 12, 'ncdim%lat': 10000} -> [12, 10000]
-            # chunks={'ncdim%time': 12, 'ncdim%lat': "20MB"} -> [12, "20MB"]
-            # chunks={'ncdim%time': 12, 'latitude': -1} -> [12, -1]
-            # chunks={'ncdim%time': 12, 'Y': None} -> [12, None]
-            # chunks={'ncdim%time': 12, 'ncdim%lat': (30, 90)} -> [12, (30, 90)]
-            # chunks={'ncdim%time': 12, 'ncdim%lat': None, 'X': 5} -> [12, None]
-            attributes = g["variable_attributes"]
-            chunks2 = []
-            for ncdim in g["variable_dimensions"][ncvar]:
-                key = f"ncdim%{ncdim}"
-                if key in chunks:
-                    chunks2.append(chunks[key])
-                    continue
-
-                found_coord_attr = False
-                dim_coord_attrs = attributes.get(ncdim)
-                if dim_coord_attrs is not None:
-                    for attr in ("standard_name", "axis"):
-                        key = dim_coord_attrs.get(attr)
-                        if key in chunks:
-                            found_coord_attr = True
-                            chunks2.append(chunks[key])
-                            break
-
-                if not found_coord_attr:
-                    # Use default chunks for this dimension
-                    chunks2.append(default_chunks)
-
-            chunks = chunks2
-
-        return chunks
-
     def _customise_field_ancillaries(self, parent_ncvar, f):
         """Create customised field ancillary constructs.
 
@@ -926,6 +624,7 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         aggregation_instructions = g["cfa_aggregation_instructions"]
         variable_attributes = g["variable_attributes"]
 
+        # Loop round aggregation instruction terms
         out = {}
         for x in self._parse_x(
             ncvar,
@@ -940,10 +639,16 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 # Already processed this term
                 continue
 
-            array = g["variables"][term_ncvar][...]
-            aggregation_instructions[term_ncvar] = self._cfa_conform_array(
-                array
+            variable = g["variables"][term_ncvar]
+            array = cfdm.netcdf_indexer(
+                variable,
+                mask=True,
+                unpack=True,
+                always_masked_array=False,
+                orthogonal_indexing=False,
+                copy=False,
             )
+            aggregation_instructions[term_ncvar] = array[...]
 
             if term == "file":
                 # Find URI substitutions that may be stored in the
@@ -967,43 +672,3 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
 
         g["cfa_aggregated_data"][ncvar] = out
         return out
-
-    def _cfa_conform_array(self, array):
-        """Conform an array so that it is suitable for CFA processing.
-
-        .. versionadded: 3.15.0
-
-        :Parameters:
-
-            array: `np.ndarray`
-                The array to conform.
-
-        :Returns:
-
-            array: `np.ndarray`
-                The conformed array.
-
-        """
-        if isinstance(array, str):
-            # string
-            return np.array(array, dtype=f"S{len(array)}").astype("U")
-
-        kind = array.dtype.kind
-        if kind == "O":
-            # string
-            return array.astype("U")
-
-        if kind in "SU":
-            # char
-            if kind == "U":
-                array = array.astype("S")
-
-            array = netCDF4.chartostring(array)
-            shape = array.shape
-            array = np.array([x.rstrip() for x in array.flat], dtype="S")
-            array = np.reshape(array, shape)
-            array = np.ma.masked_where(array == b"", array)
-            return array.astype("U")
-
-        # number
-        return array
