@@ -3,6 +3,7 @@ from os import remove
 import cfdm
 import dask.array as da
 import numpy as np
+from cfdm.data.dask_utils import cfdm_to_memory
 
 from .netcdfread import NetCDFRead
 
@@ -103,8 +104,10 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
 
                         raise ValueError(
                             f"Can't write {cfvar!r} as a CFA-netCDF "
-                            "aggregation variable. Consider setting "
-                            "cfa={'strict': False}"
+                            "aggregation variable. Possible reasons for this "
+                            "include 1) there is more than one Dask chunk "
+                            "per fragment, and 2) data values have been "
+                            "changed relative to those in the fragments."
                         )
 
                     return cfa_get_write
@@ -464,7 +467,7 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
         ):
             f_ncdim = f"f_{ncdim}"
             if f_ncdim not in g["dimensions"]:
-                # Create a new fragement dimension
+                # Create a new fragment dimension
                 self._write_dimension(f_ncdim, None, size=size)
 
             fragment_ncdimensions.append(f_ncdim)
@@ -576,55 +579,6 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
             },
         )
 
-    def _convert_to_builtin_type(self, x):
-        """Convert a non-JSON-encodable object to a JSON-encodable
-        built-in type.
-
-        Possible conversions are:
-
-        ==============  =============  ======================================
-        Input object    Output object  numpy data types covered
-        ==============  =============  ======================================
-        numpy.bool_     bool           bool
-        numpy.integer   int            int, int8, int16, int32, int64, uint8,
-                                       uint16, uint32, uint64
-        numpy.floating  float          float, float16, float32, float64
-        ==============  =============  ======================================
-
-        .. versionadded:: 3.0.0
-
-        :Parameters:
-
-            x:
-
-        :Returns:
-
-            'int' or `float` or `bool`
-
-        **Examples:**
-
-        >>> type(_convert_to_builtin_type(numpy.bool_(True)))
-        bool
-        >>> type(_convert_to_builtin_type(numpy.array([1.0])[0]))
-        double
-        >>> type(_convert_to_builtin_type(numpy.array([2])[0]))
-        int
-
-        """
-        if isinstance(x, np.bool_):
-            return bool(x)
-
-        if isinstance(x, np.integer):
-            return int(x)
-
-        if isinstance(x, np.floating):
-            return float(x)
-
-        raise TypeError(
-            f"{type(x)!r} object can't be converted to a JSON serializable "
-            f"type: {x!r}"
-        )
-
     def _check_valid(self, array, cfvar=None, attributes=None):
         """Checks for array values outside of the valid range.
 
@@ -792,7 +746,10 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
             # dimensions, with one value per fragment. If a chunk has
             # more than one unique value then the fragment's value is
             # missing data.
-            dx = data.to_dask_array()
+            #
+            # '_cfa_unique' has its own call to 'cfdm_to_memory', so
+            # we can set '_force_to_memory=False'.
+            dx = data.to_dask_array(_force_to_memory=False)
             dx_ind = tuple(range(dx.ndim))
             out_ind = dx_ind
             dx = da.blockwise(
@@ -850,6 +807,8 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
                 data if there is not a unique value.
 
         """
+        a = cfdm_to_memory(a)
+
         out_shape = (1,) * a.ndim
         a = np.unique(a)
         if np.ma.isMA(a):
@@ -918,17 +877,17 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
             if len(file_details) != 1:
                 if file_details:
                     raise ValueError(
-                        "Can't write CFA-netCDF aggregation variable from "
-                        f"{cfvar!r} when the "
-                        f"dask storage chunk defined by indices {indices} "
-                        "spans two or more files"
+                        f"Can't write {cfvar!r} as a CFA-netCDF "
+                        "aggregation variable: Dask chunk defined by index "
+                        f"{indices} spans two or more fragments. "
+                        "A possible fix for this is to set chunks=None as "
+                        "an argument of a prior call to cf.read"
                     )
 
                 raise ValueError(
-                    "Can't write CFA-netCDF aggregation variable from "
-                    f"{cfvar!r} when the "
-                    f"dask storage chunk defined by indices {indices} spans "
-                    "zero files"
+                    f"Can't write {cfvar!r} as a CFA-netCDF "
+                    "aggregation variable: Dask chunk defined by index "
+                    f"{indices} spans zero fragments."
                 )
 
             filenames, addresses, formats = file_details.pop()
@@ -1003,7 +962,10 @@ class NetCDFWrite(cfdm.read_write.netcdf.NetCDFWrite):
         # Create the location array
         # ------------------------------------------------------------
         dtype = np.dtype(np.int32)
-        if max(data.to_dask_array().chunksize) > np.iinfo(dtype).max:
+        if (
+            max(data.to_dask_array(_force_to_memory=False).chunksize)
+            > np.iinfo(dtype).max
+        ):
             dtype = np.dtype(np.int64)
 
         ndim = data.ndim
