@@ -2,7 +2,6 @@ import logging
 import os
 import tempfile
 from glob import glob
-from numbers import Integral
 from os.path import isdir
 from re import Pattern
 from urllib.parse import urlparse
@@ -58,21 +57,28 @@ def read(
     select_options=None,
     follow_symlinks=False,
     mask=True,
+    unpack=True,
     warn_valid=False,
-    chunks="auto",
+    dask_chunks="storage-aligned",
+    store_hdf5_chunks=True,
     domain=False,
     cfa=None,
+    netcdf_backend=None,
+    storage_options=None,
+    cache=True,
+    chunks="auto",
 ):
     """Read field or domain constructs from files.
 
-    The following file formats are supported: CF-netCDF, CFA-netCDF,
-    CDL, PP and UM fields datasets.
+    The following file formats are supported: netCDF, CFA-netCDF, CDL,
+    UM fields file, and PP.
 
     Input datasets are mapped to constructs in memory which are
     returned as elements of a `FieldList` or if the *domain* parameter
     is True, a `DomainList`.
 
-    NetCDF files may be on disk or on an OPeNDAP server.
+    NetCDF files may be on disk, on an OPeNDAP server, or in an S3
+    object store.
 
     Any amount of files of any combination of file types may be read.
 
@@ -408,14 +414,13 @@ def read(
             parameter.
 
         mask: `bool`, optional
-            If False then do not mask by convention when reading the
-            data of field or metadata constructs from disk. By default
-            data is masked by convention.
+            If True (the default) then mask by convention the data of
+            field and metadata constructs.
 
-            The masking by convention of a netCDF array depends on the
-            values of any of the netCDF variable attributes
-            ``_FillValue``, ``missing_value``, ``valid_min``,
-            ``valid_max`` and ``valid_range``.
+            A netCDF array is masked depending on the values of any of
+            the netCDF attributes ``_FillValue``, ``missing_value``,
+            ``_Unsigned``, ``valid_min``, ``valid_max``, and
+            ``valid_range``.
 
             The masking by convention of a PP or UM array depends on
             the value of BMDI in the lookup header. A value other than
@@ -427,6 +432,16 @@ def read(
 
             .. versionadded:: 3.4.0
 
+        unpack: `bool`, optional
+            If True, the default, then unpack arrays by convention
+            when the data is read from disk.
+
+            Unpacking is determined by netCDF conventions for the
+            following variable attributes: ``add_offset``,
+            ``scale_factor``, and ``_Unsigned``.
+
+            .. versionadded:: 1.11.2.0
+
         warn_valid: `bool`, optional
             If True then print a warning for the presence of
             ``valid_min``, ``valid_max`` or ``valid_range`` properties
@@ -543,80 +558,201 @@ def read(
 
             .. versionadded:: 1.5
 
-        chunks: `str`, `int`, `None`, or `dict`, optional
-                Specify the `dask` chunking of dimensions for data in
-                the input files.
+        dask_chunks: `str`, `int`, `None`, or `dict`, optional
+            Specify the Dask chunking for data. May be one of the
+            following:
 
-                By default, ``'auto'`` is used to specify the array
-                chunking, which uses a chunk size in bytes defined by
-                the `cf.chunksize` function, preferring square-like
-                chunk shapes across all data dimensions.
+            * ``'storage-aligned'``
 
-                If *chunks* is a `str` then each data array uses this
-                chunk size in bytes, preferring square-like chunk
-                shapes across all data dimensions. Any string value
-                accepted by the *chunks* parameter of the
-                `dask.array.from_array` function is permitted.
+              This is the default. The Dask chunk size in bytes will
+              be as close as possible the size given by
+              `cf.chunksize`, favouring square-like chunk shapes,
+              with the added restriction that the entirety of each
+              storage chunk must also lie within exactly one Dask
+              chunk.
 
-                *Parameter example:*
-                  A chunksize of 2 MiB may be specified as
-                  ``'2097152'`` or ``'2 MiB'``.
+              When reading the data from disk, an entire storage chunk
+              will be read once per Dask storage chunk that contains
+              any part of it, so ensuring that a storage chunk lies
+              within only one Dask chunk can increase performance by
+              reducing the amount of disk access (particularly when
+              the data are stored remotely to the client).
 
-                If *chunks* is `-1` or `None` then for each there is no
-                chunking, i.e. every data array has one chunk
-                regardless of its size.
+              For instance, consider a file variable that has an array
+              of 64-bit floats with shape (400, 300, 60) and a storage
+              chunk shape of (100, 5, 60), giving 240 storage chunks
+              each of size 100*5*60*8 bytes = 0.23 MiB. Then:
 
-                If *chunks* is a positive `int` then each data array
-                dimension has chunks with this number of elements.
+              * If `cf.chunksize` returned 134217728 (i.e. 128 MiB),
+                then the storage-aligned Dask chunks will have shape
+                (400, 300, 60), giving 1 Dask chunk with size of 54.93
+                MiB (compare with a Dask chunk shape of (400, 300, 60)
+                and size 54.93 MiB, if *dask_chunks* were ``'auto'``.)
 
-                If *chunks* is a `dict`, then each of its keys
-                identifies dimension in the file, with a value that
-                defines the chunking for that dimension whenever it is
-                spanned by data.
+              * If `cf.chunksize` returned 33554432 (i.e. 32 MiB),
+                then the storage-aligned Dask chunks will have shape
+                (200, 260, 60), giving 4 Dask chunks with a maximum
+                size of 23.80 MiB (compare with a Dask chunk shape of
+                (264, 264, 60) and maximum size 31.90 MiB, if
+                *dask_chunks* were ``'auto'``.)
 
-                Each dictionary key identifies a file dimension in one
-                of three ways: 1. the netCDF dimension name, preceded
-                by ``ncdim%`` (e.g. ``'ncdim%lat'``); 2. the "standard
-                name" attribute of a CF-netCDF coordinate variable
-                that spans the dimension (e.g. ``'latitude'``); or
-                3. the "axis" attribute of a CF-netCDF coordinate
-                variable that spans the dimension (e.g. ``'Y'``).
+              * If `cf.chunksize` returned 4194304 (i.e. 4 MiB),
+                then the storage-aligned Dask chunks will have shape
+                (100, 85, 60), giving 16 Dask chunks with a maximum
+                size of 3.89 MiB (compare with a Dask chunk shape of
+                (93, 93, 60) and maximum size 3.96 MiB, if
+                *dask_chunks* were ``'auto'``.)
 
-                The dictionary values may be `str`, `int` or `None`,
-                with the same meanings as those types for the *chunks*
-                parameter but applying only to the specified
-                dimension. A `tuple` or `list` of integers that sum to
-                the dimension size may also be given.
+              There are, however, some occasions when, for particular
+              data arrays in the file, the ``'auto'`` option will
+              automatically be used instead of storage-aligned Dask
+              chunks. This occurs when:
 
-                Not specifying a file dimension in the dictionary is
-                equivalent to it being defined with a value of
-                ``'auto'``.
+              * The data array in the file is stored contiguously.
 
-                *Parameter example:*
-                  ``{'T': '0.5 MiB', 'Y': [36, 37], 'X': None}``
+              * The data array in the file is compressed by convention
+                (e.g. ragged array representations, compression by
+                gathering, subsampled coordinates, etc.). In this case
+                the Dask chunks are for the uncompressed data, and so
+                cannot be aligned with the storage chunks of the
+                compressed array in the file.
 
-                *Parameter example:*
-                  If a netCDF file contains dimensions ``time``,
-                  ``z``, ``lat`` and ``lon``, then ``{'ncdim%time':
-                  12, 'ncdim%lat', None, 'ncdim%lon': None}`` will
-                  ensure that all ``time`` axes have a chunksize of
-                  12; and all ``lat`` and ``lon`` axes are not
-                  chunked; and all ``z`` axes are chunked to comply as
-                  closely as possible with the default chunks size.
+            * ``'storage-exact'``
 
-                  If the netCDF also contains a ``time`` coordinate
-                  variable with a ``standard_name`` attribute of
-                  ``'time'`` and an ``axis`` attribute of ``'T'``,
-                  then the same chunking could be specified with
-                  either ``{'time': 12, 'ncdim%lat', None, 'ncdim%lon':
-                  None}`` or ``{'T': 12, 'ncdim%lat', None,
-                  'ncdim%lon': None}``.
+              Each Dask chunk will contain exactly one storage chunk
+              and each storage chunk will lie within exactly one Dask
+              chunk.
 
-                .. note:: The *chunks* parameter is ignored for PP and
-                          UM fields files, for which the chunking is
-                          pre-determined by the file format.
+              For instance, consider a file variable that has an array
+              of 64-bit floats with shape (400, 300, 60) and a storage
+              chunk shape of (100, 5, 60) (i.e. there are 240 storage
+              chunks, each of size 0.23 MiB). Then the storage-exact
+              Dask chunks will also have shape (100, 5, 60) giving 240
+              Dask chunks with a maximum size of 0.23 MiB.
 
-                .. versionadded:: 3.14.0
+              There are, however, some occasions when, for particular
+              data arrays in the file, the ``'auto'`` option will
+              automatically be used instead of storage-exact Dask
+              chunks. This occurs when:
+
+              * The data array in the file is stored contiguously.
+
+              * The data array in the file is compressed by convention
+                (e.g. ragged array representations, compression by
+                gathering, subsampled coordinates, etc.). In this case
+                the Dask chunks are for the uncompressed data, and so
+                cannot be aligned with the storage chunks of the
+                compressed array in the file.
+
+            * ``auto``
+
+              The Dask chunk size in bytes will be as close as
+              possible to the size given by `cf.chunksize`,
+              favouring square-like chunk shapes. This may give
+              similar Dask chunk shapes as the ``'storage-aligned'``
+              option, but without the guarantee that each storage
+              chunk will lie within exactly one Dask chunk.
+
+            * A byte-size given by a `str`
+
+              The Dask chunk size in bytes will be as close as
+              possible to the given byte-size, favouring square-like
+              chunk shapes. Any string value, accepted by the *chunks*
+              parameter of the `dask.array.from_array` function is
+              permitted.
+
+              *Example:*
+                A Dask chunksize of 2 MiB may be specified as
+                ``'2097152'`` or ``'2 MiB'``.
+
+            * `-1` or `None`
+
+              There is no Dask chunking, i.e. every data array has one
+              Dask chunk regardless of its size.
+
+            * Positive `int`
+
+              Every dimension of all Dask chunks has this number of
+              elements.
+
+              *Example:*
+                For 3-dimensional data, *dask_chunks* of `10` will
+                give Dask chunks with shape (10, 10, 10).
+
+            * `dict`
+
+              Each of dictionary key identifies a file dimension, with
+              a value that defines the Dask chunking for that
+              dimension whenever it is spanned by a data array. A file
+              dimension is identified in one of three ways:
+
+              1. the netCDF dimension name, preceded by ``ncdim%``
+                (e.g. ``'ncdim%lat'``);
+
+              2. the value of the "standard name" attribute of a
+                 CF-netCDF coordinate variable that spans the
+                 dimension (e.g. ``'latitude'``);
+
+              3. the value of the "axis" attribute of a CF-netCDF
+                 coordinate variable that spans the dimension
+                 (e.g. ``'Y'``).
+
+              The dictionary values may be a byte-size string,
+              ``'auto'``, `int` or `None`, with the same meanings as
+              those types for the *dask_chunks* parameter itself, but
+              applying only to the specified dimension. In addition, a
+              dictionary value may be a `tuple` or `list` of integers
+              that sum to the dimension size.
+
+              Not specifying a file dimension in the dictionary is
+              equivalent to it being defined with a value of
+              ``'auto'``.
+
+              *Example:*
+                ``{'T': '0.5 MiB', 'Z': 'auto', 'Y': [36, 37], 'X':
+                None}``
+
+              *Example:*
+                If a netCDF file contains dimensions ``time``, ``z``,
+                ``lat`` and ``lon``, then ``{'ncdim%time': 12,
+                'ncdim%lat', None, 'ncdim%lon': None}`` will ensure
+                that, for all applicable data arrays, all ``time``
+                axes have a `dask` chunksize of 12; all ``lat`` and
+                ``lon`` axes are not `dask` chunked; and all ``z``
+                axes are `dask` chunked to comply as closely as
+                possible with the default `dask` chunk size.
+
+                If the netCDF file also contains a ``time`` coordinate
+                variable with a "standard_name" attribute of
+                ``'time'`` and an "axis" attribute of ``'T'``, then
+                the same `dask` chunking could be specified with
+                either ``{'time': 12, 'ncdim%lat', None, 'ncdim%lon':
+                None}`` or ``{'T': 12, 'ncdim%lat', None, 'ncdim%lon':
+                None}``.
+
+              .. versionadded:: 1.11.2.0
+
+        store_hdf5_chunks: `bool`, optional
+            If True (the default) then store the HDF5 chunking
+            strategy for each returned data array. The HDF5 chunking
+            strategy is then accessible via an object's
+            `nc_hdf5_chunksizes` method. When the HDF5 chunking
+            strategy is stored, it will be used when the data is
+            written to a new netCDF4 file with `cf.write` (unless
+            the strategy was modified prior to writing).
+
+            If False, or if the file being read is not in netCDF4
+            format, then no HDF5 chunking strategy is stored.
+            (i.e. an `nc_hdf5_chunksizes` method will return `None`
+            for all `Data` objects). In this case, when the data is
+            written to a new netCDF4 file, the HDF5 chunking strategy
+            will be determined by `cf.write`.
+
+            See the `cf.write` *hdf5_chunks* parameter for details
+            on how the HDF5 chunking strategy is determined at the
+            time of writing.
+
+            .. versionadded:: 1.11.2.0
 
         domain: `bool`, optional
             If True then return only the domain constructs that are
@@ -652,18 +788,91 @@ def read(
               A dictionary whose key/value pairs define text
               substitutions to be applied to the fragment file
               names. Each key may be specified with or without the
-              ``${...}`` syntax. For instance, the following are
-              equivalent: ``{'base': 'sub'}``, ``{'${base}': 'sub'}``.
-              The substitutions are used in conjunction with, and take
-              precedence over, any that are stored in the CFA-netCDF
-              file by the ``substitutions`` attribute of the ``file``
-              CFA aggregation instruction variable.
+              ``${*}`` syntax (where `*` represents any amount of any
+              characters). For instance, ``{'substitution':
+              'replacement'}`` and ``{'${substitution}': 'replacement'}``'
+              are equivalent. The substitutions are used in
+              conjunction with, and take precedence over, any that are
+              stored in the CFA-netCDF file by the ``substitutions``
+              attribute of the ``file`` fragement array variable.
 
               *Example:*
-                ``{'base': 'file:///data/'}``
+                ``{'replacement': 'file:///data/'}``
 
             .. versionadded:: 3.15.0
 
+        netcdf_backend: `None` or `str`, optional
+            Specify which library to use for reading netCDF files. By
+            default, or if `None`, then the first one of `netCDF4` and
+            `h5netcdf` to successfully open the file netCDF file is
+            used. Setting *netcdf_backend* to one of ``'netCDF4'`` and
+            ``'h5netcdf'`` will force the use of that library.
+
+            .. note:: The *netcdf_backend* parameter does not affect
+                      the opening of netCDF fragment files that define
+                      the data of aggregation variables. For these, it
+                      is always the case that the first one of
+                      `netCDF4` and `h5netcdf` to successfully open
+                      the file is used.
+
+            .. versionadded:: 1.11.2.0
+
+        storage_options: `dict` or `None`, optional
+            Pass parameters to the backend file system driver, such as
+            username, password, server, port, etc. How the storage
+            options are interpreted depends on the location of the
+            file:
+
+            **Local File System**
+
+            Storage options are ignored for local files.
+
+            **HTTP(S)**
+
+            Storage options are ignored for files available across the
+            network via OPeNDAP.
+
+            **S3-compatible services**
+
+            The backend used is `s3fs`, and the storage options are
+            used to initialise an `s3fs.S3FileSystem` file system
+            object. By default, or if `None`, then *storage_options*
+            is taken as ``{}``.
+
+            If the ``'endpoint_url'`` key is not in *storage_options*,
+            nor in a dictionary defined by the ``'client_kwargs'`` key
+            (both of which are the case when *storage_options* is
+            `None`), then one will be automatically inserted for
+            accessing an S3 file. For example, for a file name of
+            ``'s3://store/data/file.nc'``, an ``'endpoint_url'`` key
+            with value ``'https://store'`` would be created. To
+            disable this, set ``'endpoint_url'`` to `None`.
+
+            *Parameter example:*
+              For a file name of ``'s3://store/data/file.nc'``, the
+              following are equivalent: ``None``, ``{}``,
+              ``{'endpoint_url': 'https://store'}``, and
+              ``{'client_kwargs': {'endpoint_url': 'https://store'}}``
+
+            *Parameter example:*
+              ``{'key': 'scaleway-api-key...', 'secret':
+              'scaleway-secretkey...', 'endpoint_url':
+              'https://s3.fr-par.scw.cloud', 'client_kwargs':
+              {'region_name': 'fr-par'}}``
+
+            .. versionadded:: 1.11.2.0
+
+        cache: `bool`, optional
+            If True, the default, then cache the first and last array
+            elements of metadata constructs (not field constructs) for
+            fast future access. In addition, the second and
+            penultimate array elements will be cached from coordinate
+            bounds when there are two bounds per cell. For remote
+            data, setting *cache* to False may speed up the parsing of
+            the file.
+
+            .. versionadded:: 1.11.2.0
+
         umversion: deprecated at version 3.0.0
             Use the *um* parameter instead.
 
@@ -680,7 +889,10 @@ def read(
             Use methods on the returned `FieldList` instead.
 
         chunk: deprecated at version 3.14.0
-            Use the *chunks* parameter instead.
+            Use the *dask_chunks* parameter instead.
+
+        chunks: deprecated at version 1.11.2.0
+            Use the *dask_chunks* parameter instead.
 
     :Returns:
 
@@ -745,7 +957,7 @@ def read(
         _DEPRECATION_ERROR_FUNCTION_KWARGS(
             "cf.read",
             {"follow_symlinks": follow_symlinks},
-            "Use keyword 'followlink' instead.",
+            "Use keyword 'followlinks' instead.",
             removed_at="4.0.0",
         )  # pragma: no cover
 
@@ -761,7 +973,16 @@ def read(
         _DEPRECATION_ERROR_FUNCTION_KWARGS(
             "cf.read",
             {"chunk": chunk},
-            "Use keyword 'chunks' instead.",
+            "Use keyword 'dask_chunks' instead.",
+            version="3.14.0",
+            removed_at="5.0.0",
+        )  # pragma: no cover
+
+    if chunks != "auto":
+        _DEPRECATION_ERROR_FUNCTION_KWARGS(
+            "cf.read",
+            {"chunk": chunk},
+            "Use keyword 'dask_chunks' instead.",
             version="3.14.0",
             removed_at="5.0.0",
         )  # pragma: no cover
@@ -770,13 +991,6 @@ def read(
     if isinstance(select, (str, Query, Pattern)):
         select = (select,)
 
-    # Check chunks
-    if chunks is not None and not isinstance(chunks, (str, Integral, dict)):
-        raise ValueError(
-            "'chunks' parameter must be of type str, int, None or dict. "
-            f"Got: {chunks!r}"
-        )
-
     # Manage input parameters where contradictions are possible:
     if cdl_string and fmt:
         if fmt == "CDL":
@@ -885,8 +1099,8 @@ def read(
         file_glob = os.path.expanduser(os.path.expandvars(file_glob))
 
         scheme = urlparse(file_glob).scheme
-        if scheme in ("https", "http"):
-            # Do not glob a URL
+        if scheme in ("https", "http", "s3"):
+            # Do not glob a remote URL
             files2 = (file_glob,)
         else:
             # Glob files on disk
@@ -949,12 +1163,17 @@ def read(
                 um=um,
                 extra=extra,
                 height_at_top_of_model=height_at_top_of_model,
-                chunks=chunks,
+                dask_chunks=dask_chunks,
+                store_hdf5_chunks=store_hdf5_chunks,
                 mask=mask,
+                unpack=unpack,
                 warn_valid=warn_valid,
                 select=select,
                 domain=domain,
                 cfa_options=cfa_options,
+                netcdf_backend=netcdf_backend,
+                storage_options=storage_options,
+                cache=cache,
             )
 
             # --------------------------------------------------------
@@ -1064,11 +1283,16 @@ def _read_a_file(
     extra=None,
     height_at_top_of_model=None,
     mask=True,
+    unpack=True,
     warn_valid=False,
-    chunks="auto",
+    dask_chunks="storage-aligned",
+    store_hdf5_chunks=True,
     select=None,
     domain=False,
     cfa_options=None,
+    netcdf_backend=None,
+    storage_options=None,
+    cache=True,
 ):
     """Read the contents of a single file into a field list.
 
@@ -1090,6 +1314,9 @@ def _read_a_file(
         mask: `bool`, optional
             See `cf.read` for details.
 
+        unpack: `bool`, optional
+            See `cf.read` for details.
+
         verbose: `int` or `str` or `None`, optional
             See `cf.read` for details.
 
@@ -1104,6 +1331,21 @@ def _read_a_file(
 
             .. versionadded:: 3.15.0
 
+        storage_options: `dict` or `None`, optional
+            See `cf.read` for details.
+
+            .. versionadded:: 1.11.2.0
+
+        netcdf_backend: `str` or `None`, optional
+            See `cf.read` for details.
+
+            .. versionadded:: 1.11.2.0
+
+        cache: `bool`, optional
+            See `cf.read` for details.
+
+            .. versionadded:: 1.11.2.0
+
     :Returns:
 
         `FieldList` or `DomainList`
@@ -1134,7 +1376,6 @@ def _read_a_file(
             umversion = float(str(umversion).replace(".", "0", 1))
 
     extra_read_vars = {
-        "chunks": chunks,
         "fmt": selected_fmt,
         "ignore_read_error": ignore_read_error,
         "cfa_options": cfa_options,
@@ -1175,8 +1416,14 @@ def _read_a_file(
                 warnings=warnings,
                 extra_read_vars=extra_read_vars,
                 mask=mask,
+                unpack=unpack,
                 warn_valid=warn_valid,
                 domain=domain,
+                storage_options=storage_options,
+                netcdf_backend=netcdf_backend,
+                dask_chunks=dask_chunks,
+                store_hdf5_chunks=store_hdf5_chunks,
+                cache=cache,
             )
         except MaskError:
             # Some data required for field interpretation is missing,
