@@ -391,6 +391,72 @@ class DomainTest(unittest.TestCase):
             np.allclose(latitude_specific.array - y_points_specific, 0)
         )
 
+    def test_Domain_del_construct(self):
+        """Test the `del_construct` Domain method."""
+        # Test a domain without cyclic axes. These are equivalent tests to
+        # those in the cfdm test suite, to check behaviour is the same in cf.
+        d = self.d.copy()
+
+        self.assertIsInstance(
+            d.del_construct("dimensioncoordinate1"), cf.DimensionCoordinate
+        )
+        self.assertIsInstance(
+            d.del_construct("auxiliarycoordinate1"), cf.AuxiliaryCoordinate
+        )
+        with self.assertRaises(ValueError):
+            d.del_construct("auxiliarycoordinate1")
+
+        self.assertIsNone(
+            d.del_construct("auxiliarycoordinate1", default=None)
+        )
+
+        self.assertIsInstance(d.del_construct("measure:area"), cf.CellMeasure)
+
+        # NOTE: this test will fail presently because of a bug which means
+        # that Field.domain doesn't inherit the cyclic() axes of the
+        # corresponding Field (see Issue #762) which will be fixed shortly.
+        #
+        # Test a domain with cyclic axes, to ensure the cyclic() set is
+        # updated accordingly if a cyclic axes is the one removed.
+        e = cf.example_field(2).domain  # this has a cyclic axes 'domainaxis2'
+        # To delete a cyclic axes, must first delete this dimension coordinate
+        # because 'domainaxis2' spans it.
+        self.assertIsInstance(
+            e.del_construct("dimensioncoordinate2"), cf.DimensionCoordinate
+        )
+        self.assertEqual(e.cyclic(), set(("domainaxis2",)))
+        self.assertIsInstance(e.del_construct("domainaxis2"), cf.DomainAxis)
+        self.assertEqual(e.cyclic(), set())
+
+    def test_Domain_cyclic_iscyclic(self):
+        """Test the `cyclic` and `iscyclic` Domain methods."""
+        # A field and its domain should have the same cyclic() output.
+        f1 = cf.example_field(1)  # no cyclic axes
+        d1 = f1.domain
+        f2 = cf.example_field(2)  # one cyclic axis, 'domainaxis2' ('X')
+        d2 = f2.domain
+
+        # Getting
+        self.assertEqual(d1.cyclic(), f1.cyclic())
+        self.assertEqual(d1.cyclic(), set())
+        self.assertFalse(d1.iscyclic("X"))
+        self.assertFalse(d1.iscyclic("Y"))
+        self.assertFalse(d1.iscyclic("Z"))
+        self.assertFalse(d1.iscyclic("T"))
+        self.assertEqual(d2.cyclic(), f2.cyclic())
+        self.assertEqual(d2.cyclic(), set(("domainaxis2",)))
+        self.assertTrue(d2.iscyclic("X"))
+        self.assertFalse(d2.iscyclic("Y"))
+        self.assertFalse(d2.iscyclic("Z"))
+        self.assertFalse(d2.iscyclic("T"))
+
+        # Setting
+        self.assertEqual(d2.cyclic("X", iscyclic=False), set(("domainaxis2",)))
+        self.assertEqual(d2.cyclic(), set())
+        self.assertEqual(d2.cyclic("X", period=360), set())
+        self.assertEqual(d2.cyclic(), set(("domainaxis2",)))
+        self.assertTrue(d2.iscyclic("X"))
+
 
 if __name__ == "__main__":
     print("Run date:", datetime.datetime.now())
