@@ -30,8 +30,8 @@ def _formula_terms(standard_name):
     )
 
     aux = cf.AuxiliaryCoordinate()
-    aux.long_name = "Computed from parametric {} vertical coordinates".format(
-        standard_name
+    aux.long_name = (
+        f"Computed from parametric {standard_name} vertical coordinates"
     )
 
     if standard_name == "atmosphere_ln_pressure_coordinate":
@@ -731,10 +731,8 @@ def _formula_terms(standard_name):
 
     else:
         raise ValueError(
-            "Bad standard name: {}, "
-            "not an element of FormulaTerms.standard_names".format(
-                standard_name
-            )
+            f"Bad standard name: {standard_name}, "
+            "not an element of FormulaTerms.standard_names"
         )
 
     return (field, aux, computed_standard_name)
@@ -748,18 +746,27 @@ class FormulaTermsTest(unittest.TestCase):
         f = cf.example_field(1)
         self.assertIsNone(f.auxiliary_coordinate("altitude", default=None))
 
-        g = f.compute_vertical_coordinates(verbose=None)
-
-        altitude = g.auxiliary_coordinate("altitude")
         orog = f.domain_ancillary("surface_altitude")
         a = f.domain_ancillary("ncvar%a")
         b = f.domain_ancillary("ncvar%b")
 
+        # Set bounds properties on 'a' to check that they do not
+        # appear on the bounds of the computed vertical coordinates
+        a.bounds.set_property("foo", "bar")
+
+        g = f.compute_vertical_coordinates(verbose=None)
+        altitude = g.auxiliary_coordinate("altitude")
+
         self.assertTrue(altitude)
         self.assertTrue(altitude.has_bounds())
         self.assertEqual(altitude.shape, (1,) + orog.shape)
         self.assertEqual(altitude.bounds.shape, altitude.shape + (2,))
 
+        # Check that bounds properties were cleared
+        properties = altitude.bounds.properties()
+        properties.pop("units", None)
+        self.assertFalse(properties)
+
         # Check array values
         orog = orog.data.insert_dimension(-1)
         x = a.data + b.data * orog
@@ -881,6 +888,7 @@ class FormulaTermsTest(unittest.TestCase):
         for standard_name in cf.formula_terms.FormulaTerms.standard_names:
             if standard_name == "atmosphere_hybrid_height_coordinate":
                 continue
+
             f, a, csn = _formula_terms(standard_name)
 
             g = f.compute_vertical_coordinates(verbose=None)
@@ -889,13 +897,8 @@ class FormulaTermsTest(unittest.TestCase):
 
             self.assertTrue(
                 x.equals(a, atol=1e-5, rtol=1e-05, verbose=-1),
-                "{}, {}, {}\n{}\n{}".format(
-                    standard_name,
-                    x.array,
-                    a.array,
-                    x.bounds.array,
-                    a.bounds.array,
-                ),
+                f"{standard_name}, {x.array}, {a.array}"
+                f"\n{x.bounds.array}\n{a.bounds.array}",
             )
 
 
